-- Haskell XML Toolkit
module Hxt where


-- import Text.XML.HXT.Arrow


data NTree a  = NTree a [NTree a]  -- rose tree
 
data XNode    = XText String  -- plain text node
              | ...
              | XTag QName XmlTrees 
                -- element name and list of attributes
              | XAttr QName  -- attribute name
              | ...
 
type QName    = ...  -- qualified name
 
type XmlTree  = NTree XNode
 
type XmlTrees = [XmlTree]


type XmlFilter = XmlTree -> [XmlTree]

type Filter a b = a -> [b]

isA  :: (a -> Bool) -> (a -> [a])
isA p x
  | p x       = [x]
  | otherwise = []
  
isXText		              :: XmlFilter
isXText t@(NTree (XText _) _) =  [t]
isXText _                     =  []


trans	:: XmlTree -> XmlTree
trans t	= exp(t)
 
ftrans	:: XmlTree -> [XmlTree]
ftrans t = [exp(t)]

part	:: XmlTree -> XmlTree
part t
  | p t =  expr(t)
  | otherwise = error "f not defined"
 
fpart	:: XmlFilter
fpart t
  | p t = [expr(t)]
  | otherwise = []
  
  
(>>>)	    :: XmlFilter -> XmlFilter -> XmlFilter
(f >>> g) t =  concat [g t' | t' <- f t]

(<+>)	    :: XmlFilter -> XmlFilter -> XmlFilter 
(f <+> g) t =  f t ++ g t

orElse	:: XmlFilter -> XmlFilter -> XmlFilter
orElse f g t
  | null res1 = g t
  | otherwise = res1
  where res1 = f t

when	:: XmlFilter -> XmlFilter -> XmlFilter
when f g t
  | null (g t) = [t]
  | otherwise  = f t
 
guards	:: XmlFilter -> XmlFilter -> XmlFilter
guards g f t
  | null (g t) = []
  | otherwise  = f t

deep	:: XmlFilter -> XmlFilter
deep f  = f `orElse` (getChildren >>> deep f)

multi	:: XmlFilter -> XmlFilter
multi f  = f <+> (getChildren >>> multi f)
