

module Statechart where


{-- Deep and shallow embeddings of expressions --}

data Expr a = Expr a :+ Expr a
            | Expr a :- Expr a
            | Expr a :* Expr a
            | Expr a :/ Expr a
            | A a

type IntExpr = Expr Integer

runD :: IntExpr -> Integer
runD e = case e of
  e1 :+ e2 -> runD e1 + runD e2
  e1 :- e2 -> runD e1 - runD e2
  e1 :* e2 -> runD e1 * runD e2
  e1 :/ e2 -> runD e1 `div` runD e2
  A i -> i
  
value1 = runD $ A 3 :* (A 4 :+ A 8)  

negate :: Num a=> Expr a -> Expr a
negate x = A (fromInteger (-1)) :* x

summate :: [Expr a] -> Expr a
summate (x:xs) = foldl (:+) x xs

value2 = runD $ summate [A i | i <- [1 .. 100]]

negate' x = -x

summate' :: [Integer] -> Integer
summate' = sum



{-- END --}



type Event = String
data State = St String [(Event, State)]

instance Show State where
  show (St s es) = "<"++s++">"
  
chart1 :: State
chart1 = 
  let st_a = St "a" []
      st_b = St "b" [("x", st_a), ("y", st_a)]
      st_c = St "c" [("z", st_b), ("y", st_a)]
  in st_c

chart2 :: State
chart2 = 
  let st_a = St "a" [("w", st_c)]
      st_b = St "b" [("x", st_a), ("y", st_a)]
      st_c = St "c" [("z", st_b), ("y", st_a)]
  in st_c

run :: State -> [Event] -> (State, [Event])
run s@(St a sux) es = case es of
  [] -> (s, [])
  (e:es') -> case lookup e sux of
               Nothing -> (s, es)
               Just s' -> run s' es'
  
  
